"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getRepositoryConfigFile = getRepositoryConfigFile;
const errors_1 = require("@oclif/core/lib/errors");
const core_1 = require("@oclif/core");
const path_1 = require("./path");
/**
 * @returns The parsed connectrc.js file.
 */
async function getRepositoryConfigFile() {
    const configPath = (0, path_1.getCustomerPath)("connectrc.js");
    try {
        return require(configPath);
    }
    catch (error) {
        // To get ux.warn to render properly, we need to stop the action first
        core_1.ux.action.stop("💥");
        if (error.code === "MODULE_NOT_FOUND") {
            throw new errors_1.CLIError(`💥 Could not find a connectrc.js file in the root directory!\nExpected to be at: ${configPath}`, {
                code: error.code,
                suggestions: [
                    "Make sure you have a connectrc.js file in the root directory of your repository.",
                    `If your root directory is not the same as your current working directory,
you can set the \`REPOSITORY_ROOT_DIRECTORY\` environment variable as a relative path
pointing to the correct connectrc.js directory.`,
                ],
            });
        }
        core_1.ux.warn(error.stack);
        if (error.name?.includes("SyntaxError")) {
            throw new errors_1.CLIError("💥 Syntax error found in your connectrc.js file!", {
                code: "SYNTAX_ERROR",
            });
        }
        throw new errors_1.CLIError("💥 Could not load your connectrc.js file! \nTake a look at the original error printed above.", {
            code: error.code,
        });
    }
}
