"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getCustomerPath = getCustomerPath;
exports.cleanupTemplatePath = cleanupTemplatePath;
exports.cleanupFixturePath = cleanupFixturePath;
exports.cleanupExternalDataPath = cleanupExternalDataPath;
/* eslint-disable @typescript-eslint/prefer-nullish-coalescing */
const path_1 = require("path");
const os_1 = __importDefault(require("os"));
/**
 * @returns The provided path prepended with the root directory.
 */
function getCustomerPath(path) {
    return (0, path_1.join)(process.cwd(), process.env.REPOSITORY_ROOT_DIRECTORY || "", path);
}
/**
 * Cleans up the template path, removing the root directory and the 'src/templates' prefix.
 *
 * This is needed to support the use of absolute paths in the templates, since those will
 * be added later on when reading the files.
 */
function cleanupTemplatePath(path) {
    return cleanupDelimiters(cleanupAbsolutePath(path).replace((0, path_1.join)("src", "templates"), ""));
}
/**
 * Cleans up the fixture path, removing the root directory and the 'src/fixtures' prefix.
 *
 * This is needed to support the use of absolute paths in the templates, since those will
 * be added later on when reading the files.
 */
function cleanupFixturePath(path) {
    return cleanupDelimiters(cleanupAbsolutePath(path).replace((0, path_1.join)("src", "fixtures"), ""));
}
/**
 * Cleans up the external data path, removing the root directory and the 'src/external-data' prefix.
 *
 * This is needed to support the use of absolute paths in the templates, since those will
 * be added later on when reading the files.
 */
function cleanupExternalDataPath(path) {
    return cleanupDelimiters(cleanupAbsolutePath(path).replace((0, path_1.join)("src", "external-data"), ""));
}
function cleanupAbsolutePath(path) {
    if (!path) {
        return "";
    }
    const replacedPath = path.replace(process.cwd(), "");
    return cleanupDelimiters(replacedPath);
}
function cleanupDelimiters(path) {
    const platformFileDelimiter = os_1.default.platform() === "win32" ? "\\" : "/";
    let replacedPath = path.replace(process.cwd(), "");
    if (replacedPath.startsWith(`${platformFileDelimiter}${platformFileDelimiter}`)) {
        replacedPath = replacedPath.slice(2);
    }
    return replacedPath;
}
