"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterConnectionsByTemplate = filterConnectionsByTemplate;
const path_1 = __importDefault(require("path"));
const config_1 = require("../customer/config");
/**
 * Returns all connections that are associated with a template by checking
 * the config file.
 */
async function filterConnectionsByTemplate(templatePath, connections) {
    const config = await (0, config_1.getRepositoryConfigFile)();
    const prefix = path_1.default.join("src", "templates");
    let fullPath = templatePath;
    if (!templatePath.startsWith(prefix)) {
        fullPath = path_1.default.join(prefix, templatePath);
    }
    // All connection ids that references this template
    const configConnectionIds = config.environments.reduce((connectionIds, environment) => {
        for (const templateConfig of environment.templates) {
            const templatePaths = Object.values(templateConfig.paths)
                .map((filePath) => path_1.default.normalize(path_1.default.resolve(filePath)))
                .map((filePath) => filePath.replace(/\\/g, "/")); // this is needed to check the path in all OS
            // environment doesn't even target this template
            if (!templatePaths.includes(path_1.default.resolve(fullPath).replace(/\\/g, "/"))) {
                continue;
            }
            for (const id of templateConfig.connection_ids) {
                connectionIds.add(id);
            }
        }
        return connectionIds;
    }, new Set());
    const filteredConnections = connections.filter((connection) => configConnectionIds.has(connection.id));
    return filteredConnections.map((connection) => {
        return {
            name: connection.name,
            value: connection,
        };
    });
}
